require_relative '../../spec_helper.rb'

require 'date'

module Contraption
  describe Header do
    before(:each) do
      @title = "The Title"
      @date = DateTime.now
      @type = :a_type
      @tags = []
      @summary = "some witty line to get people to read the post"
      @h = Header.new title: @title, publication_date: @date, type: @type, tags: @tags, summary: @summary
    end

    context "initialization" do

      it "returns the initial title" do
        @h.title.should eq(@title)
      end

      it "returns the initial date" do
        @h.publication_date.should eq(@date)
      end

      it "returns the initial type" do
        @h.type.should eq(@type)
      end

      it "returns the initial tags" do
        @h.tags.should eq(@tags)
      end

      it "returns the summary" do
        @h.summary.should eq(@summary)
      end
    end

    context ".new?" do
      it "knows when it is new" do
        h = Header.from "published: now"
        h.new?.should be_true
      end

      it "knows when it is not new" do
        h = Header.from "published: #{@date}"
        h.new?.should be_false
      end
    end

    context "#filename" do
      it "stores the filename if specified in the initializer" do
        h = Header.new ({filename: "a-filename.md"})
        h.filename.should eq "a-filename.md"
      end

      it "returns a filename computed from the title if no filename given" do
        h = Header.new ({title: "A Filename"})
        h.filename.should eq "a-filename.md"
      end

      it "removes non-alphanumeric characters" do
        h = Header.new ({title: "Call Me ... Maybe?!"})
        h.filename.should eq "call-me-maybe.md"
      end

      it "does not remove '-'" do
        h = Header.new ({title: "Hyphenated-title"})
        h.filename.should eq "hyphenated-title.md"
      end

      it "has only one extension" do
        h = Header.new ({filename: "a-filename.md"})
        h.filename.split('.').length.should eq 2
      end

      it "has an extension when not specified" do
        h = Header.new ({filename: "a-filename"})
        h.filename.split('.').length.should eq 2
      end
    end

    context "#update" do
      it "returns a new object" do
        update_h = @h.update
        update_h.should_not eq @h
      end

      it "does not modify the original header" do
        initial_h = @h
        @h.update
        initial_h.should eq @h
      end

      it "stores the new title" do
        new_h = @h.update title: "New Title"
        new_h.title.should eq "New Title"
      end

      it "retains the original values" do
        new_h = @h.update
        new_h.title.should eq @h.title
      end

      it "stores all of the new values" do
        new_h = @h.update title: "New Title", tags: %w[ruby]
        new_h.title.should eq "New Title"
        new_h.tags.should  eq %w[ruby]
      end
    end

    context "::from" do
      let(:title)         { "The Title" }
      let(:pub_date)      { "Published: Mon, 31 Dec 2012 15:41:00 EST" }
      let(:expected_date) { DateTime.new(2012, 12, 31, 15, 41, 00, '-5') }
      let(:summary)       { "SuMMAry: Something witty to get 'eyeballs'" }
      let(:type)          { "tyPe: Article" }
      let(:tags)          { "TaGS: red, green, blue, purple" }

      it "returns a Header" do
        h = Header.from ""
        h.class.should eq Header
      end

      it "finds the title" do
        h = Header.from title
        h.title.should eq title
      end

      it "finds the filename" do
        h = Header.from "filenAmE: the-filename-goes-here.md"
        h.filename.should eq "the-filename-goes-here.md"
      end

      it "finds the publication date" do
        h = Header.from pub_date
        h.publication_date.should eq expected_date
      end

      it "does not manipulate now" do
        h = Header.from "published: now"
        h.publication_date.should eq :now
      end

      it "finds the summary" do
        h = Header.from summary
        h.summary.should eq "Something witty to get 'eyeballs'"
      end

      it "finds the type" do
        h = Header.from type
        h.type.should eq :article
      end

      it "finds the tags" do
        h = Header.from tags
        h.tags.map{|t| t.to_s}.should eq %w[red green blue purple]
      end

      it "finds multiple elements" do
        combined_input = [title, pub_date, type, tags].join("\n")
        h = Header.from combined_input
        h.title.should eq title
        h.publication_date.should eq expected_date
        h.type.should eq :article
        h.tags.map{|t| t.to_s}.should eq %w[red green blue purple]
      end

      it "alternate definition for titles" do
        title = "thetitle: and a subtitle"
        h = Header.from("title: " + title)
        h.title.should eq title
      end

      it "handles initial whitespace" do
        t = "\t A title starting with Whitespace"
        h = Header.from "title: " + t
        h.title.should eq "A title starting with Whitespace"
      end

      it "handles unsupported header lines" do
        h = Header.from "unsupported: this should not exist"
        expect {h.unsupported}.to raise_error NameError
      end

      it "ignores '====' separators" do
        example = [title, "="*15].join("\n")
        h = Header.from example
        h.title.should eq title
      end

      it "finds external links" do
        h = Header.from "external: http://ruby-lang.org"
        h.external.should eq "http://ruby-lang.org"
      end
    end
  end
end
