require_relative '../../spec_helper'

module Contraption
  describe Location do
    before(:all) { `mkdir /tmp/qscfr` }
    after(:all)  { `rm -r /tmp/qscfr` }

    let (:dir) { "/tmp/qscfr" }
    after(:each) { `rm -rf #{dir}/*` }

    let (:l) { Location.new dir }

    it "has a path" do
      l.path.should eq dir
    end

    it "can create itself on disk" do
      new_location = Location.new(dir + '/newdir')
      new_location.create!
      File.exist?(dir+'/newdir').should be_true
    end

    context "#list" do
      it "handles empty directories" do
        l.list.should eq []
      end

      it "handles directories with one file" do
        `touch #{dir}/monkeys`
        l.list.should eq ['monkeys']
      end

      it "handles directories with multiple files" do
        10.times {|i| `touch #{dir}/#{i}`}
        l.list.should eq (0..9).map{|i| i.to_s}
      end

      it "only finds files with specific extension" do
        `touch #{dir}/10.txt #{dir}/10.html #{dir}/10.md #{dir}/10.rb`
        l.list(".html").should eq ["10.html"]
      end

      it "finds files with any of a list of extensions" do
        `touch #{dir}/10.txt #{dir}/10.html #{dir}/10.md #{dir}/10.rb`
        l.list(%w[.html .md]).should eq %w[10.html 10.md]
      end
    end

    context "#read" do
      let(:content) { "stuff\nthings" }
      before(:each) { File.open("#{dir}/file", "w:UTF-8") { |f| f.print content } }

      it "returns the contents of the file" do
        l.read("file").should eq content
      end

      it "yields to block when passed" do
        l.read("file") do |io|
          io.map {|l| l[0]}
        end.should eq ["s","t"]
      end

      it "returns a useful token if file does not exist" do
        l.read("nonexistent_file").should eq :file_does_not_exist
      end

      it "returns a useful token if file is not readable" do
        `touch #{dir}/file && chmod -r #{dir}/file`
        l.read("#{dir}/file").should eq :file_not_readable
      end
    end

    context "#write" do
      it "writes the provided string to a file" do
        l.write "filename", "this is a test string.\nand another line just for fun."
      end

      it "does not destructively write to a file" do
        `touch #{dir}/a_new_file`
        l.write("a_new_file", "some content that will not be written").should eq :file_already_exists
      end
    end

    context "#remove" do
      it "forgets an existing file" do
        l.write("a_new_file", "some stuff to write")
        l.remove("a_new_file")
        l.list.should_not include("a_new_file")
      end

      it "does nothing for non-existing files" do
        l.remove("a_non_existant_file").should eq :file_does_not_exist
      end
    end

    context "directory trees" do
      it "recursively creates directories" do
        l.write "some/thing", "something"
        l.read("some/thing").should eq "something"
      end

      it "recursively lists files" do
        l.write "some/thing", "something"
        l.write "a/new/file", "a"
        l.list.should eq %w[a new file some thing]
      end
    end

    context "equality" do
      it "returns true for Locations with the same path" do
        l.==(Location.new(dir)).should be_true
      end

      it "returns false for Locations with different paths" do
        l.==(Location.new(dir+"a")).should be_false
      end
    end

    context "interrogation" do
      it "contains exist?" do
        l.exist?
      end

      it "contains readable?" do
        l.readable?
      end

      it "contains writable?" do
        l.writable?
      end

      it "contains executable?" do
        l.executable?
      end
    end

    context "cd" do
      it "returns a new Location rooted at argument" do
        `mkdir #{dir}/subdir`
        l.cd("subdir").path.should eq "#{dir}/subdir"
      end

      it "creates location if directory does not exist" do
       l.cd("subdir").exist?.should be_true
      end
    end
  end
end
